//
//  BLEScannerClass.swift
//  NEO2
//
//  Created by Michael Ruvalcaba on 2/15/24.
//

import Foundation
import SwiftUI
import CoreBluetooth

struct DiscoveredPeripheral {
    // Struct to represent a discovered peripheral
    var peripheral: CBPeripheral
    var advertisedData: String
}

let SPS_SERVICE_3300 = "1820" // VP3300
let SPS_SERVICE_3320 = "49535343-FE7D-4AE5-8FA9-9FAFD205E455" // VP3320
let SPS_SERVICE_3350 = "0783b03e-8535-b5a0-7140-a304d2495cb7" //"0783b03e-8535-b5a0-7140-a304d2495cb7" // VP3350
let SPS_SERVICE_6800 = "A002" // VP6800

@objc class BluetoothScanner: NSObject, CBCentralManagerDelegate, ObservableObject {
    @Published var discoveredPeripherals = [DiscoveredPeripheral]()
    @objc @Published var isScanning = false
    @objc var centralManager: CBCentralManager!
    @objc var discoveredPeriphs: Dictionary<CBPeripheral, String> = Dictionary()
    // Set to store unique peripherals that have been discovered
    @objc public var discoveredPeripheralSet = Set<CBPeripheral>()
    //    var timer: Timer?
    
    
    
    
    override init() {
        super.init()
        centralManager = CBCentralManager(delegate: self, queue: nil)
    }
    
    @objc public func startScan() {
        if centralManager.state == .poweredOn {
            // Set isScanning to true and clear the discovered peripherals list
            isScanning = true
            discoveredPeripherals.removeAll()
            discoveredPeripheralSet.removeAll()
            objectWillChange.send()
            
            let serviceUUIDs: [CBUUID] = [
                CBUUID(string: SPS_SERVICE_6800),
                CBUUID(string: SPS_SERVICE_3300),
                CBUUID(string: SPS_SERVICE_3320),
                CBUUID(string: SPS_SERVICE_3350)
            ]
            
            // Start scanning for peripherals
            centralManager.scanForPeripherals(withServices: serviceUUIDs)
            
            // Start a timer to stop and restart the scan every 2 seconds
            //            timer?.invalidate()
            //            timer = Timer.scheduledTimer(withTimeInterval: 2.0, repeats: true) { [weak self] timer in
            //                self?.centralManager.stopScan()
            //                self?.centralManager.scanForPeripherals(withServices: serviceUUIDs, options: nil)
            //            }
        }
    }
    
    @objc public func stopScan() {
        // Set isScanning to false and stop the timer
        isScanning = false
        //        timer?.invalidate()
        centralManager.stopScan()
    }
    
    @objc public func centralManagerDidUpdateState(_ central: CBCentralManager) {
        switch central.state {
        case .unknown:
            //print("central.state is .unknown")
            stopScan()
        case .resetting:
            //print("central.state is .resetting")
            stopScan()
        case .unsupported:
            //print("central.state is .unsupported")
            stopScan()
        case .unauthorized:
            //print("central.state is .unauthorized")
            stopScan()
        case .poweredOff:
            //print("central.state is .poweredOff")
            stopScan()
        case .poweredOn:
            //print("central.state is .poweredOn")
            return
        @unknown default:
            print("central.state is unknown")
        }
    }
    
    @objc public func centralManager(_ central: CBCentralManager, didDiscover peripheral: CBPeripheral, advertisementData: [String : Any], rssi RSSI: NSNumber) {
        // Build a string representation of the advertised data and sort it by names
        var advertisedData = advertisementData.map { "\($0): \($1)" }.sorted(by: { $0 < $1 }).joined(separator: "\n")
        
        // Convert the timestamp into human readable format and insert it to the advertisedData String
        let timestampValue = advertisementData["kCBAdvDataTimestamp"] as! Double
        // print(timestampValue)
        let dateFormatter = DateFormatter()
        dateFormatter.dateFormat = "HH:mm:ss"
        let dateString = dateFormatter.string(from: Date(timeIntervalSince1970: timestampValue))
        let uuid = peripheral.identifier.uuidString
        
        advertisedData = "actual rssi: \(RSSI) dB\n" + "Timestamp: \(dateString)\n" + advertisedData + "\nkUUID: \(uuid)"
        
        // If the peripheral is not already in the list
        if !discoveredPeripheralSet.contains(peripheral) {
            // Add it to the list and the set
            discoveredPeripherals.append(DiscoveredPeripheral(peripheral: peripheral, advertisedData: advertisedData))
            discoveredPeripheralSet.insert(peripheral)
            discoveredPeriphs.updateValue(advertisedData, forKey: peripheral)
            objectWillChange.send()
        } else {
            // If the peripheral is already in the list, update its advertised data
            if let index = discoveredPeripherals.firstIndex(where: { $0.peripheral == peripheral }) {
                discoveredPeripherals[index].advertisedData = advertisedData
                discoveredPeriphs.updateValue(advertisedData, forKey: peripheral)
                objectWillChange.send()
            }
        }
    }
}
