//
//  FirstViewController.swift
//  NEO2_Test_App
//
//  Created by IDTech on 7/14/16.
//  Copyright © 2016 IDTech. All rights reserved.
//

import UIKit
import MobileCoreServices
import UniformTypeIdentifiers

var _lcdDisplayMode: Int32 = 0

extension Data {
    struct HexEncodingOptions: OptionSet {
        let rawValue: Int
        static let upperCase = HexEncodingOptions(rawValue: 1 << 0)
    }
    
    func hexEncodedString(options: HexEncodingOptions = []) -> String {
        let format = options.contains(.upperCase) ? "%02hhX" : "%02hhx"
        return map { String(format: format, $0) }.joined()
    }
}

extension String {
	func displayEntries() -> String{
		return self.replacingOccurrences(of: "AnyHashable(", with: "\n").replacingOccurrences(of: "):", with: ":")
	}
}

class FirstViewController: UIViewController, IDT_NEO2_Delegate, IDT_Device_Delegate, UIDocumentPickerDelegate, UIAlertViewDelegate, UIDocumentMenuDelegate {
    
    var connection = Connection()

    @IBOutlet weak var connectionStatus: UILabel!
    @IBOutlet weak var resultsTextView: UITextView!
    @IBOutlet weak var dataTextView: UITextView!
    @IBOutlet weak var friendlyName: UITextField!
    @IBOutlet weak var rkiName: UITextField!

    
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        
        IDT_NEO2.sharedController().delegate = self
        IDT_Device.sharedController().delegate = self
        
        let tbvc = tabBarController as! TabBarController
        connection = tbvc.connection
    }
    
    override func viewWillAppear(_ animated: Bool) {
        connectionStatus.text = connection.getTheStatus()
        connectionStatus.backgroundColor = connection.getTheBackgroundColor()
        resultsTextView.text = connection.getTheResultsText()
        dataTextView.text = connection.getTheDataText()
        friendlyName.text = IDT_NEO2.sharedController().device_getBLEFriendlyName()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func setConnectionStatus(_ status: String, backgroundColor: UIColor) {
        connection.setTheStatus(status)
        DispatchQueue.main.async {
            self.connectionStatus.text = self.connection.getTheStatus()
            self.connection.setTheBackgroundColor(backgroundColor)
            self.connectionStatus.backgroundColor = self.connection.getTheBackgroundColor()
        }
        
        
        NotificationCenter.default.post(name: Notification.Name(rawValue: "load"), object: nil)
    }
    
    public func deviceConnected() {
        setConnectionStatus("Connected", backgroundColor: UIColor.green)
        
        connection.appendMessageToResultsField("NEO2 Connected\nFramework Version: \(IDT_Device.sdk_version())", field: resultsTextView)
    }
    
    func deviceDisconnected() {
        setConnectionStatus("Disconnected", backgroundColor: UIColor.red)
    }
    
    func deviceMessage(_ message: String!) {
        connection.appendMessageToResultsField(message, field: resultsTextView)

    }
    
    @IBAction func getSDKVer()
    {
        deviceSDKverion()
    }
    
    func deviceSDKverion() {
        let sdkVersion = IDT_Device.sdk_version()
        dataTextView.text = sdkVersion
    }
    
    func data(inOutMonitor data: Data!, incoming isIncoming: Bool) {
        if isIncoming {
            connection.appendMessageToDataField("IN: \(data.hexEncodedString())\n", field: dataTextView)
        } else {
            connection.appendMessageToDataField("\n====================\nOUT: \(data.hexEncodedString())\n", field: dataTextView)
        }
    }
    
    func swipeMSRData(_ cardData: IDTMSRData!) {
		NSLog("--MSR event received, type: \(cardData.event), data: \(cardData.encTrack1)")
		//NSLog("YO! card data", cardData.encryptedTags);
		switch cardData.event {
		case EVENT_MSR_CARD_DATA:
			switch cardData.captureEncodeType {
			case CAPTURE_ENCODE_TYPE_ISOABA:
				connection.appendMessageToResultsField("Encode Type: ISO/ABA", field: resultsTextView)
			case CAPTURE_ENCODE_TYPE_AAMVA:
				connection.appendMessageToResultsField("Encode Type: AA/MVA", field: resultsTextView)
			case CAPTURE_ENCODE_TYPE_Other:
				connection.appendMessageToResultsField("Encode Type: Other", field: resultsTextView)
			case CAPTURE_ENCODE_TYPE_Raw:
				connection.appendMessageToResultsField("Encode Type: Raw", field: resultsTextView)
			case CAPTURE_ENCODE_TYPE_JIS_I:
				connection.appendMessageToResultsField("Encode Type: CAPTURE_ENCODE_TYPE_JIS_I", field: resultsTextView)
			case CAPTURE_ENCODE_TYPE_JIS_II:
				connection.appendMessageToResultsField("Encode Type: CAPTURE_ENCODE_TYPE_JIS_II", field: resultsTextView)
			default:
				connection.appendMessageToResultsField("Encode Type: UNKWOWN", field: resultsTextView)
			}
			
			switch cardData.captureEncryptType {
			case CAPTURE_ENCRYPT_TYPE_AES:
				connection.appendMessageToResultsField("Encrypt Type: AES", field: resultsTextView)
			case CAPTURE_ENCRYPT_TYPE_TDES:
				connection.appendMessageToResultsField("Encrypt Type: TDES", field: resultsTextView)
			case CAPTURE_ENCRYPT_TYPE_NO_ENCRYPTION:
				connection.appendMessageToResultsField("Encrypt Type: NONE", field: resultsTextView)
			default:
				connection.appendMessageToResultsField("Encrypt Type: UNKNOWN", field: resultsTextView)
			}
			
			connection.appendMessageToResultsField("Full card data: \(cardData.cardData == nil ? "N/A" : cardData.cardData.hexEncodedString())", field: resultsTextView)
			connection.appendMessageToResultsField("Track 1: \(cardData.track1 == nil ? "N/A" : cardData.track1)", field: resultsTextView)
			connection.appendMessageToResultsField("Track 2: \(cardData.track2 == nil ? "N/A" : cardData.track2)", field: resultsTextView)
			connection.appendMessageToResultsField("Track 3: \(cardData.track3 == nil ? "N/A" : cardData.track3)", field: resultsTextView)
			connection.appendMessageToResultsField("Length Track 1: \(cardData.track1Length)", field: resultsTextView)
			connection.appendMessageToResultsField("Length Track 2: \(cardData.track2Length)", field: resultsTextView)
			connection.appendMessageToResultsField("Length Track 3: \(cardData.track3Length)", field: resultsTextView)
			connection.appendMessageToResultsField("Encoded Track 1: \(cardData.encTrack1 == nil ? "N/A" : cardData.encTrack1.hexEncodedString())", field: resultsTextView)
			connection.appendMessageToResultsField("Encoded Track 2: \(cardData.encTrack2 == nil ? "N/A" : cardData.encTrack2.hexEncodedString())", field: resultsTextView)
			connection.appendMessageToResultsField("Encoded Track 3: \(cardData.encTrack3 == nil ? "N/A" : cardData.encTrack3.hexEncodedString())", field: resultsTextView)
			connection.appendMessageToResultsField("Hash Track 1: \(cardData.hashTrack1 == nil ? "N/A" : cardData.hashTrack1.hexEncodedString())", field: resultsTextView)
			connection.appendMessageToResultsField("Hash Track 2: \(cardData.hashTrack2 == nil ? "N/A" : cardData.hashTrack2.hexEncodedString())", field: resultsTextView)
			connection.appendMessageToResultsField("Hash Track 3: \(cardData.hashTrack3 == nil ? "N/A" : cardData.hashTrack3.hexEncodedString())", field: resultsTextView)
			connection.appendMessageToResultsField("KSN: \(cardData.ksn == nil ? "N/A" : cardData.ksn.hexEncodedString())", field: resultsTextView)
			connection.appendMessageToResultsField("\nSessionID: \(cardData.sessionID == nil ? "N/A" : cardData.sessionID.hexEncodedString())", field: resultsTextView)
			connection.appendMessageToResultsField("\nReader Serial Number: \(cardData.rsn == nil ? "N/A" : cardData.rsn)", field: resultsTextView)
			connection.appendMessageToResultsField("\nRead Status: \(cardData.readStatus)", field: resultsTextView)
			
			if cardData.unencryptedTags != nil {
				connection.appendMessageToResultsField("Unencrypted Tags: \( cardData.unencryptedTags.description.displayEntries())", field: resultsTextView)
			}
			
			if cardData.encryptedTags != nil {
				connection.appendMessageToResultsField("Encrypted Tags: \(cardData.encryptedTags.description.displayEntries())", field: resultsTextView)
			}
			
			if cardData.maskedTags != nil {
				connection.appendMessageToResultsField("Masked Tags: \(cardData.maskedTags.description.displayEntries())", field: resultsTextView)
			}
			
			NSLog("Track 1: \(cardData.track1 == nil ? "N/A" : cardData.track1)")
			NSLog("Track 2: \(cardData.track2 == nil ? "N/A" : cardData.track2)")
			NSLog("Track 3: \(cardData.track3 == nil ? "N/A" : cardData.track3)")
			NSLog("Encoded Track 1: \(cardData.encTrack1 == nil ? "N/A" : cardData.encTrack1.hexEncodedString())")
			NSLog("Encoded Track 2: \(cardData.encTrack2 == nil ? "N/A" : cardData.encTrack2.hexEncodedString())")
			NSLog("Encoded Track 3: \(cardData.encTrack3 == nil ? "N/A" : cardData.encTrack3.hexEncodedString())")
			NSLog("Hash Track 1: \(cardData.hashTrack1 == nil ? "N/A" : cardData.hashTrack1.hexEncodedString())")
			NSLog("Hash Track 2: \(cardData.hashTrack2 == nil ? "N/A" : cardData.hashTrack2.hexEncodedString())")
			NSLog("Hash Track 3: \(cardData.hashTrack3 == nil ? "N/A" : cardData.hashTrack3.hexEncodedString())")
			NSLog("SessionID: \(cardData.sessionID == nil ? "N/A" : cardData.sessionID.hexEncodedString())")
			NSLog("nReader Serial Number: \(cardData.rsn == nil ? "N/A" : cardData.rsn)")
			NSLog("Read Status: \(cardData.readStatus)")
			NSLog("KSN: \(cardData.ksn == nil ? "N/A" : cardData.ksn.hexEncodedString())")
			
		case EVENT_MSR_CANCEL_KEY:
			connection.appendMessageToResultsField("(Event) MSR Cancel Key received: \(cardData.encTrack1)", field: resultsTextView)
			
		case EVENT_MSR_BACKSPACE_KEY:
			connection.appendMessageToResultsField("(Event) MSR Backspack Key received: \(cardData.encTrack1)", field: resultsTextView)
			
		case EVENT_MSR_ENTER_KEY:
			connection.appendMessageToResultsField("(Event) MSR Enter Key received: \(cardData.encTrack1)", field: resultsTextView)
			
		case EVENT_MSR_UNKNOWN:
			connection.appendMessageToResultsField("(Event) MSR unknown event, data: \(cardData.encTrack1)", field: resultsTextView)
		case EVENT_MSR_TIMEOUT:
			connection.appendMessageToResultsField("(Event) MSR Timeout",field: resultsTextView)
			
		default:
			break
			
		}
    }
    
	@objc func completeEMV() {
        NotificationCenter.default.post(name: Notification.Name(rawValue: "completeEMV"), object: nil)
    }
    
	@objc func startEMV() {
        NotificationCenter.default.post(name: Notification.Name(rawValue: "startEMV"), object: nil)
    }
    
    func emvTransactionData(_ emvData: IDTEMVData!, errorCode error: Int32) {
        var loopData = connection.getTheStressTestEMVLoopStatus()
        
        connection.appendMessageToResultsField("EMV transaction data response: \(IDT_NEO2.sharedController().device_getResponseCodeString(error))\n", field: resultsTextView)
        
        if emvData == nil {
            connection.appendMessageToResultsField("EMV TRANSACTION ERROR. Refer to EMV_RESULT_CODE_V2_response = \(error)", field: resultsTextView)
            return;
        }
        
		if emvData.resultCodeV2 == EMV_RESULT_CODE_V2_TIME_OUT {
			
		}
		
		if emvData.resultCodeV2 != EMV_RESULT_CODE_V2_NO_RESPONSE {
			connection.appendMessageToResultsField("EMV_RESULT_CODE_V2_RESPONSE: \(emvData.resultCodeV2.rawValue)", field: resultsTextView)
		}
		
		if emvData.resultCodeV2 == EMV_RESULT_CODE_V2_GO_ONLINE {
			connection.appendMessageToResultsField("ONLINE REQUEST", field: resultsTextView)
			loopData = false
			
			if connection.getTheAutoCompleteStatus() {
				Timer.scheduledTimer(timeInterval: 0.5, target: self, selector: #selector(completeEMV), userInfo: nil, repeats: false)
			}
		}
		
		if emvData.resultCodeV2 == EMV_RESULT_CODE_V2_START_TRANS_SUCCESS {
			loopData = false
			connection.appendMessageToResultsField("Start success: authentication required", field: resultsTextView)
		}
		
		if emvData.resultCodeV2 == EMV_RESULT_CODE_V2_APPROVED || emvData.resultCodeV2 == EMV_RESULT_CODE_V2_APPROVED_OFFLINE {
			connection.appendMessageToResultsField("APPROVED", field: resultsTextView);
		}
		
		if emvData.resultCodeV2 == EMV_RESULT_CODE_V2_SWIPE_NON_ICC {
			connection.appendMessageToResultsField("MSR Data Captured", field: resultsTextView)
		}
		
		if emvData.cardType == 0 {
			connection.appendMessageToResultsField("CONTACT", field: resultsTextView)
		}
		
		if emvData.cardType == 1 {
			connection.appendMessageToResultsField("CONTACTLESS", field: resultsTextView)
		}
		
		if emvData.unencryptedTags != nil {
			connection.appendMessageToResultsField("Unencrypted Tags: \(emvData.unencryptedTags.description.displayEntries())", field: resultsTextView)
		}
		
		if emvData.encryptedTags != nil {
			connection.appendMessageToResultsField("Encrypted Tags: \(emvData.encryptedTags.description.displayEntries())", field: resultsTextView)
		}
		
		if emvData.maskedTags != nil {
			connection.appendMessageToResultsField("Masked Tags: \(emvData.maskedTags.description.displayEntries())", field: resultsTextView)
		}
		
		if emvData.hasAdvise {
			connection.appendMessageToResultsField("Response has advise request", field: resultsTextView)
		}
		
		if emvData.hasReversal {
			connection.appendMessageToResultsField("Response has reversal request", field: resultsTextView)
		}

        
        if loopData {
            Timer.scheduledTimer(timeInterval: 0.5, target: self, selector: #selector(startEMV), userInfo: nil, repeats: false)
        }
    }
    
    func pinpadData(_ value: Data, keySN KSN: Data, event: EVENT_PINPAD_Types) {
        /*
         EVENT_PINPAD_UNKNOWN = 11,
         EVENT_PINPAD_ENCRYPTED_PIN,
         EVENT_PINPAD_NUMERIC,
         EVENT_PINPAD_AMOUNT,
         EVENT_PINPAD_ACCOUNT,
         EVENT_PINPAD_ENCRYPTED_DATA,
         EVENT_PINPAD_CANCEL,
         EVENT_PINPAD_TIMEOUT,
         EVENT_PINPAD_FUNCTION_KEY,
         EVENT_PINPAD_DATA_ERROR,
         EVENT_PINPAD_PAN_ERROR,
         EVENT_PINPAD_PIN_DUKPT_MISSING,
         EVENT_PINPAD_PIN_DUKPT_EXHAUSTED,
         EVENT_PINPAD_DISPLAY_MESSAGE_ERROR
         */
        switch event {
        case EVENT_PINPAD_UNKNOWN:
            connection.appendMessageToResultsField("EVENT_PINPAD_UNKNOWN", field: resultsTextView)
        case EVENT_PINPAD_ENCRYPTED_PIN:
            connection.appendMessageToResultsField("EVENT_PINPAD_ENCRYPTED_PIN", field: resultsTextView)
            connection.appendMessageToResultsField("PINBLOCK = \(value.hexEncodedString())", field: resultsTextView)
            connection.appendMessageToResultsField("KSN = \(KSN.hexEncodedString())", field: resultsTextView)

            
        case EVENT_PINPAD_NUMERIC:
            connection.appendMessageToResultsField("EVENT_PINPAD_NUMERIC", field: resultsTextView)
        case EVENT_PINPAD_AMOUNT:
            connection.appendMessageToResultsField("EVENT_PINPAD_AMOUNT", field: resultsTextView)
        case EVENT_PINPAD_ACCOUNT:
            connection.appendMessageToResultsField("EVENT_PINPAD_ACCOUNT", field: resultsTextView)
        case EVENT_PINPAD_ENCRYPTED_DATA:
            connection.appendMessageToResultsField("EVENT_PINPAD_ENCRYPTED_DATA", field: resultsTextView)
        case EVENT_PINPAD_CANCEL:
            connection.appendMessageToResultsField("EVENT_PINPAD_CANCEL", field: resultsTextView)
        case EVENT_PINPAD_TIMEOUT:
            connection.appendMessageToResultsField("EVENT_PINPAD_TIMEOUT", field: resultsTextView)
        case EVENT_PINPAD_FUNCTION_KEY:
            connection.appendMessageToResultsField("Function Key = \(value.hexEncodedString())", field: resultsTextView)
        case EVENT_PINPAD_PAN_ERROR:
            connection.appendMessageToResultsField("EVENT_PINPAD_PAN_ERROR", field: resultsTextView)
        case EVENT_PINPAD_PIN_DUKPT_MISSING:
            connection.appendMessageToResultsField("EVENT_PINPAD_PIN_DUKPT_MISSING", field: resultsTextView)
        case EVENT_PINPAD_PIN_DUKPT_EXHAUSTED:
            connection.appendMessageToResultsField("EVENT_PINPAD_PIN_DUKPT_EXHAUSTED", field: resultsTextView)
        case EVENT_PINPAD_DISPLAY_MESSAGE_ERROR:
            connection.appendMessageToResultsField("EVENT_PINPAD_DISPLAY_MESSAGE_ERROR", field: resultsTextView)
        default:
            break
        }
    }

    
    
    
    func lcdDisplay(_ mode: Int32, lines: [Any]!) {
        var str = ""
        _lcdDisplayMode = mode
		print("mode: \(mode)")
		print("lines: \(lines)")
        if lines != nil {
            for s in lines {
                str += s as! String
                str += "\n"
            }
        }
        
        switch mode {
        case 0x10:
            connection.setTheResultsText("")
            resultsTextView.text = connection.getTheResultsText()
            NotificationCenter.default.post(name: Notification.Name(rawValue: "load"), object: nil)
        case 0x03:
			connection.appendMessageToResultsField(str, field: resultsTextView)
            //connection.setTheResultsText(str)
            DispatchQueue.main.async {
                self.resultsTextView.text = self.connection.getTheResultsText()
            }
            NotificationCenter.default.post(name: Notification.Name(rawValue: "load"), object: nil)
        case 0x01, 0x02, 0x08:
            let alert = UIAlertView(title: "Please select", message: str, delegate: self, cancelButtonTitle: "Cancel", otherButtonTitles: "Ok", "")
            alert.alertViewStyle = .plainTextInput
            alert.show()
        default:
            break
        }
    }
    var _mode: EMV_PIN_MODE_Types = EMV_PIN_MODE_CANCEL;
    func pinRequest(_ mode: EMV_PIN_MODE_Types, key: Data!, pan PAN: Data!, startTO: Int32, intervalTO: Int32, language: String!) {
        _mode = mode;
        let alert = UIAlertView(title: "PIN REQUEST", message:"Please Enter PIN", delegate: self, cancelButtonTitle: "Cancel", otherButtonTitles: "Ok", "")
        alert.alertViewStyle = .secureTextInput
        alert.show()
    }
    
    func alertView(_ alertView: UIAlertView, didDismissWithButtonIndex buttonIndex: Int) {
        if (alertView.title == "Please Select") {
            if buttonIndex == 1 {
                let selectedValue = Int((alertView.textField(at: 0)?.text) ?? "") ?? 0
                alertView.textField(at: 0)?.resignFirstResponder()
                IDT_NEO2.sharedController().emv_callbackResponseLCD(_lcdDisplayMode, selection: UInt8(selectedValue))
                print("Selected Value is \(selectedValue)")
            }
            else {
                IDT_NEO2.sharedController().emv_callbackResponseLCD(0, selection: 0)
            }
        }
        if (alertView.title == "PIN REQUEST") {
            if buttonIndex == 1 {
                let PIN: String? = alertView.textField(at: 0)?.text
                alertView.textField(at: 0)?.resignFirstResponder()
                IDT_NEO2.sharedController().emv_callbackResponsePIN(_mode, ksn: nil, pin: PIN?.data(using: .utf8))
            }
            else {
                IDT_NEO2.sharedController().emv_callbackResponsePIN(EMV_PIN_MODE_CANCEL, ksn: nil, pin: nil)
            }
        }
    }
    @IBAction func searchForBLE(_ sender: UIButton){
        IDT_NEO2.sharedController().device_setBLEFriendlyName(friendlyName.text!)
        IDT_NEO2.sharedController().device_enableBLEDeviceSearch(nil);
        
    }
    
    @IBAction func searchForAnyBLE(_ sender: UIButton)
    {
        IDT_NEO2.sharedController().delegate = self
        IDT_NEO2.sharedController().scan(forBLEDevices: 2.0, serviceUUIDs: nil, options: nil)
        return
    }
    
	@IBAction func getAllConfigGroups(_ sender: UIButton) {
		

		var result: NSData?
		let rt: RETURN_CODE = IDT_NEO2.sharedController().ctls_getAllConfigGroups(&result)
		if RETURN_CODE_DO_SUCCESS == rt {
			
			let data = Data(referencing: result!)
		
			connection.appendMessageToResultsField(IDTUtility.configGroupListing(data), field: resultsTextView)
		} else {
			connection.displayReturnError("Get firmware version", rt: rt, field: resultsTextView)
		}
	}
    
    @IBAction func rkiDemo(_ sender: UIButton) {
        
        let rt: RETURN_CODE = IDT_NEO2.sharedController().device_remoteKeyInjection(true, keyName: rkiName.text, isForeground: false)
        if RETURN_CODE_DO_SUCCESS == rt {
        } else {
            connection.displayReturnError("RKI Demo Failed", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func rkiProd(_ sender: UIButton) {
        
        let rt: RETURN_CODE = IDT_NEO2.sharedController().device_remoteKeyInjection(false, keyName: rkiName.text, isForeground: false)
        if RETURN_CODE_DO_SUCCESS == rt {
        } else {
            connection.displayReturnError("RKI Prod Failed", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func resignReponders(_ sender: Any) {
        friendlyName.resignFirstResponder()
        rkiName.resignFirstResponder()
    }
    
    @IBAction func getFirmwareVersion(_ sender: UIButton) {
		

        var result: NSString?
        let rt: RETURN_CODE = IDT_NEO2.sharedController().device_getFirmwareVersion(&result)
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Get firmware version: \(result!)", field: resultsTextView)
        } else {
            connection.displayReturnError("Get firmware version", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func getL2Version(_ sender: UIButton) {
        var result: NSString?
        let rt: RETURN_CODE = IDT_NEO2.sharedController().emv_getEMVL2Version(&result)
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Get EMV L2 version: \(result!)", field: resultsTextView)
        } else {
            connection.displayReturnError("Get EMV L2 version", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func getSerialNumber(_ sender: UIButton) {
        var result: NSString?
        let rt: RETURN_CODE = IDT_NEO2.sharedController().config_getSerialNumber(&result)
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Get serial number: \(result!)", field: resultsTextView)
        } else {
            connection.displayReturnError("Get serial number", rt: rt, field: resultsTextView)
        }
		

		
    }
    
    @IBAction func getICCStatus(_ sender: UIButton) {
        var response: UnsafeMutablePointer<ICCReaderStatus>? = nil
        let rt: RETURN_CODE = IDT_NEO2.sharedController().icc_get(&response)
        
        if RETURN_CODE_DO_SUCCESS != rt {
            connection.displayReturnError("Get ICC status", rt: rt, field: resultsTextView)
            return
        }
        
        var sta: String
        
        if (response?.pointee.iccPower)! {
            sta = "[ICC powered]"
        } else {
            sta = "[ICC power not ready]"
        }
        
        if (response?.pointee.cardSeated)! {
            sta = "\(sta), [Card seated]"
        } else {
            sta = "\(sta), [Card not seated]"
        }
        
        connection.appendMessageToResultsField(sta, field: resultsTextView)
    }
    
    @IBAction func enableCTLS_MSR(_ sender: UIButton) {
        let rt: RETURN_CODE = IDT_NEO2.sharedController().msr_startMSRSwipe()
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Enable MSR: OK.", field: resultsTextView)
        } else {
            connection.displayReturnError("Enable CTLS / MSR", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func disableCTLS_MSR(_ sender: UIButton) {
        let rt: RETURN_CODE = IDT_NEO2.sharedController().msr_cancelMSRSwipe()
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Disable MSR: OK.", field: resultsTextView)
        } else {
            connection.displayReturnError("Disable CTLS / MSR", rt: rt, field: resultsTextView)
        }
    }
    

    
    @IBAction func clearOutputs(_ sender: UIBarButtonItem) {
        connection.clearOutputs(resultsTextView, dataTextView: dataTextView)
    }
    var PKoverBLE = false
    @IBAction func BTNsendPKUpdateEA(_ sender: UIButton) //Set PKoverBLE false to send over physical connection
    {
        PKoverBLE = false
        selectPKFile()
    }
    
    @IBAction func BTNsendPKUpdateBLE(_ sender:UIButton) //Set PKoverBLE true to send over BLE
    {
        PKoverBLE = true
        selectPKFile()
    }
    //In order to browse filesystem, make sure you have iCloud capability and selected 'iCloud Documents'
    func selectPKFile() //Brings up the menu to browse for the pk file on device or iCloud storage
    {
        let importMenu = UIDocumentMenuViewController(documentTypes: [String(kUTTypeData)], in: .import) //Sets up the documentPicker to look for data files (
        importMenu.delegate = self
        importMenu.modalPresentationStyle = .formSheet       
        self.present(importMenu, animated: true, completion: nil)
    }
    //These three are required to pick pk file
    public func documentPicker(_ controller: UIDocumentPickerViewController, didPickDocumentsAt urls: [URL]) {
        guard let myURL = urls.first else {
            return
        }
        let fileData = try! Data(contentsOf: myURL)
        
        print("import result : \(myURL)\r")
        print("contents : \(fileData)")
        connection.pkUpdate = true
        if PKoverBLE
        {
            IDT_Device.sharedController().sendPKUpdateBLE(fileData) //This function is for over BLE
        }
        else
        {
            IDT_Device.sharedController().sendPKUpdate(fileData) //This function is for over physical connection
        }
    }
    
    
    @objc public func documentMenu(_ documentMenu:UIDocumentMenuViewController, didPickDocumentPicker documentPicker: UIDocumentPickerViewController) {
        documentPicker.delegate = self
        present(documentPicker, animated: true, completion: nil)
    }
    
    
    func documentPickerWasCancelled(_ controller: UIDocumentPickerViewController) {
        print("view was cancelled")
        dismiss(animated: true, completion: nil)
    }
    //This is the method the SDK is looking for to use as callback
    func updateStatus(_ type: PK_STATUS_Type, currentBlock: Int32, totalBlocks: Int32, error: RETURN_CODE) {
        switch type {
        case PK_STATUS_COMPLETED:
            connection.appendMessageToDataField("PK Update Process Completed", field: resultsTextView)
            connection.pkUpdate = false
            return
        case PK_STATUS_FAILED:
            connection.appendMessageToDataField("PK Update Process Failed", field: resultsTextView)
            return
        case PK_STATUS_STARTED:
            connection.appendMessageToDataField("PK Update Process Started", field: resultsTextView)
            return
        case PK_STATUS_APPLYING_UPDATE:
            connection.appendMessageToDataField("PK Update Process Applying Update. Please wait...", field: resultsTextView)
            return
        case PK_STATUS_SENDING_BLOCK:
            connection.appendMessageToDataField(String(format: "Sending block %d of %d", currentBlock, totalBlocks), field: resultsTextView)
            return
        default:
            return
        }
    }
}

